@@ -6,12 +6,12 @@ import json
 from urllib import urlencode
 
 import requests
-from oauth_hook import OAuthHook
+from rauth.hook import OAuth1Hook
 
-from query import Resolve, Table, Submit, Facets, Flag, Geopulse, Geocode
+from query import Resolve, Table, Submit, Facets, Flag, Geopulse, Geocode, Diffs, Match, Multi
 
 API_V3_HOST = "http://api.v3.factual.com"
-DRIVER_VERSION_TAG = "factual-python-driver-1.2.3"
+DRIVER_VERSION_TAG = "factual-python-driver-1.3.0"
 
 class Factual(object):
     def __init__(self, key, secret):
@@ -28,6 +28,9 @@ class Factual(object):
     def resolve(self, values):
         return Resolve(self.api, {'values': values})
 
+    def match(self, values):
+        return Match(self.api, {'values': values})
+
     def raw_read(self, path, raw_params):
         return self.api.raw_read(path, raw_params)
 
@@ -49,8 +52,14 @@ class Factual(object):
     def monetize(self):
         return Table(self.api, 'places/monetize')
 
+    def diffs(self, table, start, end):
+        return Diffs(self.api, 't/' + table + '/diffs', start, end)
+
+    def multi(self, queries):
+        return Multi(self.api, queries).make_request()
+
     def _generate_token(self, key, secret):
-        access_token = OAuthHook(consumer_key=key, consumer_secret=secret, header_auth=True)
+        access_token = OAuth1Hook(consumer_key=key, consumer_secret=secret, header_auth=True)
         return access_token
 
 
@@ -59,67 +68,73 @@ class API(object):
         self.client = requests.session(hooks={'pre_request': access_token})
 
     def get(self, query):
-        response = self._handle_request(query.path, query.params)
+        response = self._handle_request(query.path, query.params, self.client.get)
         return response
 
     def post(self, query):
-        response = self._make_post_request(query.path, query.params)
+        response = self._handle_request(query.path, query.params, self.client.post)
         return response
         
     def schema(self, query):
-        response = self._handle_request(query.path + '/schema', query.params)
+        response = self._handle_request(query.path + '/schema', query.params, self.client.get)
         return response['view']
 
     def raw_read(self, path, raw_params):
         url = self._build_base_url(path)
-        if isinstance(raw_params, str):
-            url += raw_params
-        else:
-            url += self._make_query_string(raw_params)
-        return self._make_request(url).text
+        return self._make_request(url, raw_params, self.client.get).text
+
+    def raw_stream_read(self, path, raw_params):
+        url = self._build_base_url(path, raw_params)
+        for line in self._make_request(url, raw_params, self.client.get).iter_lines():
+            if line:
+                yield line
 
     def build_url(self, path, params):
-        url = self._build_base_url(path) + self._make_query_string(params)
+        url = self._build_base_url(path)
+        url += '?' + self._make_query_string(params)
         return url
 
+    def build_multi_url(self, query):
+        return '/' + query.path + '?' + self._make_query_string(query.params)
+
     def _build_base_url(self, path):
-        return API_V3_HOST + '/' + path + '?'
+        return API_V3_HOST + '/' + path
 
-    def _handle_request(self, path, params):
-        url = self.build_url(path, params)
-        response = self._make_request(url)
+    def _handle_request(self, path, params, request_method):
+        url = self._build_base_url(path)
+        response = self._make_request(url, params, request_method)
         payload = json.loads(response.text)
-        if payload['status'] != 'ok':
-            raise APIException(response.status_code, payload, url)
-        return payload['response']
-
-    def _make_request(self, url):
-        headers = {'X-Factual-Lib': DRIVER_VERSION_TAG}
-        response = self.client.get(url, headers=headers)
-        return response
-
-    def _make_post_request(self, path, params):
-        url = self.build_url(path, params)
-        headers = {'X-Factual-Lib': DRIVER_VERSION_TAG}
-        response = self.client.post(url, headers=headers)
-        payload = json.loads(response.text)
-        if payload['status'] != 'ok':
-            raise APIException(response.status_code, payload, url)
+        if payload['status'] == 'error':
+            raise APIException(response.status_code, payload, response.url)
         return payload['response'] if 'response' in payload else payload
 
+    def _make_request(self, url, params, request_method):
+        headers = {'X-Factual-Lib': DRIVER_VERSION_TAG}
+        request_params = self._transform_params(params)
+        response = request_method(url, headers=headers, params=request_params)
+        if not 200 <= response.status_code < 300:
+            raise APIException(response.status_code, response.text, response.url)
+        return response
+
     def _make_query_string(self, params):
+        return urlencode([(k,v) for k,v in self._transform_params(params).items()])
+
+    def _transform_params(self, params):
+        if isinstance(params, str):
+            return params
         string_params = []
         for key, val in params.items():
             transformed = json.dumps(val) if not isinstance(val, str) else val
             string_params.append((key, transformed))
-        return urlencode(string_params)
+        return dict(string_params)
+
 
 class APIException(Exception):
     def __init__(self, status_code, response, url):
         self.status_code = status_code
         self.response = response
         self.url = url
-        exception = {'http_status_code':status_code,'response':response, 'url':url}
+        exception = {'http_status_code':status_code,'response':response,'url':url,'driver_version':DRIVER_VERSION_TAG}
         Exception.__init__(self, exception)
 
     def get_status_code(self):
