@@ -1,7 +1,10 @@
 from .resolve import Resolve
+from .match import Match
 from .table import Table
 from .submit import Submit
 from .facets import Facets
 from .flag import Flag
 from .geopulse import Geopulse
 from .geocode import Geocode
+from .diffs import Diffs
+from .multi import Multi
