@@ -153,22 +153,52 @@ class FactualAPITestSuite(unittest.TestCase):
         self.assertGreaterEqual(len(result), 1)
         self.assertTrue(all(row['place_locality'] == 'Los Angeles' for row in result))
 
-#    def test_submit_without_id(self):
-#        values = {'name': 'factual', 'locality': 'los angeles', 'address': '1801 Ave of the Stars'}
-#        submit = self.factual.submit('global', values=values).user('python_driver_tester')
-#        response = submit.write()
-#        self.assertFalse(response['new_entity'])
-#
-#    def test_submit_with_id(self):
-#        values = {'name': 'Factual'}
-#        submit = self.factual.submit('global', factual_id='03c26917-5d66-4de9-96bc-b13066173c65', values=values).user('python_driver_tester')
-#        response = submit.write()
-#        self.assertFalse(response['new_entity'])
-#
-#    def test_flag(self):
-#        flag = self.factual.flag('global', '03c26917-5d66-4de9-96bc-b13066173c65').user('python_driver_tester').other().debug(True)
-#        response = flag.write()
-#        self.assertEqual('ok', response['status'])
+    def test_submit_without_id(self):
+        values = {'longitude': 100}
+        submit = self.factual.submit('2EH4Pz', values=values).user('python_driver_tester')
+        response = submit.write()
+        if 'new_entity' in response:
+            self.assertTrue(response['new_entity'])
+        else:
+            self.assertIn('status', response)
+            self.assertEqual('warning', response['status'])
+
+    def test_submit_with_id(self):
+        values = {'longitude': 100}
+        submit = self.factual.submit('2EH4Pz', factual_id='0545b03f-9413-44ed-8882-3a9a461848da', values=values).user('python_driver_tester')
+        response = submit.write()
+        if 'new_entity' in response:
+            self.assertFalse(response['new_entity'])
+        else:
+            self.assertIn('status', response)
+            self.assertEqual('warning', response['status'])
+
+
+    def test_flag(self):
+        flag = self.factual.flag('2EH4Pz', '0545b03f-9413-44ed-8882-3a9a461848da').user('python_driver_tester').other().debug(True)
+        response = flag.write()
+        self.assertEqual('ok', response['status'])
+
+    def test_diffs(self):
+        diffs = self.factual.diffs('2EH4Pz', 1339123455775, 1339136968687).data()
+        self.assertGreater(len(diffs), 0)
+
+    def test_diffs_streaming(self):
+        diff_request = self.factual.diffs('2EH4Pz', 1339123455775, 1339136968687)
+        batch = diff_request.data()
+        streamed = list(diff_request.stream())
+        self.assertItemsEqual(batch, streamed)
+
+    def test_match(self):
+        match = self.factual.match({'name':'McDonalds','address':'10451 Santa Monica Blvd','locality':'Los Angeles','region':'CA'})
+        match_id = match.get_id()
+        self.assertEqual('bd886f67-9d86-40c5-9217-f7bcd53cfc0e', match_id)
+
+    def test_multi(self):
+        q1 = self.factual.table('places').filters({'postcode':'90067'})
+        q2 = self.factual.facets('places').filters({'postcode':'90067'}).select('category')
+        response = self.factual.multi({'query1':q1,'query2':q2})
+        self.assertTrue('query1' in response and 'query2' in response)
 
 if __name__ == '__main__':
     unittest.main()
